;+
; Mission     : ASO-S/HXI
; Name        : hxi_clean
; Purpose     : make Clean image for HXI data
; Category    : imaging
; Syntax      : 
; Inputs      :
;       counts      - input data in counts, [91,ntime ]
;       exptime     - expousre time in s, [ntime]
;       patt        - pattern, [nxy,nxy,91]
;       totf        - total flux
;       psize/xc/yc/nxy - map parameters: pixel size/xcenter/ycenter/number of pixels along x(y)
;       det_id      - detector/subcollimator index mask
;       [uniformw]  - weighting of subcollimators
;       niter       - para. for Clean, number of iterations
;       gain_fac    - para. for Clean, gain factor
;
; Outputs     :
;       Clean_image - reconstructed Clean image
;
; Outputs (OPTIONAL):
; History     :
;       Aug. 2019, Yang Su, first beta version, developed for imaging simultion; 
;               image correction for pointing is added. Yang Su and Wei Chen
;       - Jun. 2022, v1.0 beta, Yang Su, many changes to the code following the development of simulated data;
;       Aug. 2022, Yang Su, added corrections for pointing changes;
;       Sep. 2022, v1.1 beta, Yang Su, a few improvements, added no_pot_corr;
;       Oct. 2022 - Apr. 2023, many changes are made to the code based on imaging test of real data.
;       May 19 2023, Yang Su, new (smaller) beamw for testing
;       Jan. 16 2023, YS, add keyword bwf, default 3; add output res_image
; Contact     : yang.su@pmo.ac.cn
;-
pro hxi_clean, counts , exptime, patt , totf=totf, psize=psize,xc=xc, yc=yc, nxy=nxy, $
  det_id=det_id,niter=niter,gain_fac=gain_fac,uniformw= uniformw,$
  bwf = bwf, no_pot_corr=no_pot_corr, res_image=res_image, Clean_image=Clean_image

  if not keyword_set(psize) then psize=1. else psize=psize[0]
  if not keyword_set(exptime) then exptime=1.
  if not keyword_set(totf) then totflux= 1. else totflux=totf/exptime  ;240109,YS
  if not keyword_set(xc) then xc=0
  if not keyword_set(yc) then yc=0
  if not keyword_set(nxy) then nxy=121  ;231223
  if not keyword_set(det_id) then det_id= indgen(91)
  if not keyword_set(niter) then niter= 1000
  if not keyword_set(uniformw) then uniformw =0
  if not keyword_set(no_pot_corr) then no_pot_corr = 0
  if not keyword_set(bwf) then bwf = 2. ;240106, YS
  
  if keyword_set(no_pot_corr) then begin
    ;check the size/dim of counts
    datasize = size(counts)
    pattsize = size(patt)
    if datasize[0] eq 1 then counts=reform(counts,datasize[1],1 )
    if datasize[0] eq 2 then counts=reform(total(counts,2),datasize[1],1 )
    ;if pattsize[0] eq 3 then ;patt=reform(patt,pattsize[1],1 )
    if pattsize[0] ge 4 then begin
      patt=reform( (patt[*,*,*,0] ),pattsize[1],pattsize[2],pattsize[3],1 )
      message, 'Warning! You select [no_pot_corr], pattern is averaged over time.',/con
    endif

  endif
  
  n_det = n_elements(det_id)
  ;if not keyword_set(sr) then beam_w=6.1/2./psize  else beam_w= sr[is]/2./psize

  ds=hxi_get_config_data()
  pitches = ds.grid_p *0.0001
  angs = (!pi/180.)*ds.grid_angles
  indc = ds.grid_c_index;con index
  inds = ds.grid_s_index;sin index
  DtL = (ds.subcoll_L*!pi/180./60./60.)
  
  sr=   atan(pitches[0:90] *1.d-2,1.2-0.00 )*180./!pi*3600./2.

  ;beam_w= sr[det_id[0]]/psize
  beam_w= sr[det_id[0]]*2./2.355 /bwf ;230519,YS, new beamsize for test; 240106; 240309
  ;beam_w= sr[det_id[0]]/3. 
  
  HXI_Backproj, counts , exptime, patt , totf=totf , psize=psize,$
    xc=xc, yc=yc, nxy=nxy, det_id=det_id, uniformw= uniformw,no_pot_corr=no_pot_corr, bp_image=bp_image
    
  if keyword_set( uniformw ) then begin
    u = DtL* cos( angs[indc])/pitches[indc]
    v = DtL* sin( angs[inds])/pitches[inds]

    sf_weight =sqrt(u^2.+v^2.)   ;uniform weight

    sf_weight =sf_weight/total(sf_weight)
    sf_weight = [rebin(sf_weight, n_elements(sf_weight)*2,/sample), sf_weight[-1]]

  endif else begin
    sf_weight = fltarr(91)+1.
  endelse
 
  ntimes= (size(counts))[2]

  the_counts = reform( counts[det_id,*], n_det*ntimes )
  the_sf_weight = reform( rebin( sf_weight[det_id],n_det, ntimes),  n_det*ntimes )
  
  if ntimes eq 1 then begin
    this_patt= reform( reform(patt[*,*,det_id,0]), (size(patt))[1],(size(patt))[2],n_det*ntimes)
  endif else begin
    this_patt= reform( patt[*,*,det_id,*], (size(patt))[1],(size(patt))[2],n_det*ntimes)
  endelse
  
  nx=(size(this_patt))[1]
  ny=(size(this_patt))[2]
  n_det_new= (size(this_patt))[3]  ;n_det*n_timeint
  
  dirty=bp_image
  dirty_now=bp_image
  clean_com=bp_image*0.
  gain_fac=0.05
 
  p_avg= average(this_patt,3)
  p_avg2=  total((this_patt - rebin(p_avg,nxy,nxy, n_det_new, /sample))^2. ,3)/(float(n_det_new)-1)

  for jj=0, niter,1  do begin
    maxp=where(dirty_now eq max(dirty_now))
    maxp=maxp[0]
    if abs(min(dirty_now)) lt max(dirty_now) then begin
    ;if max(dirty_now) ge max(dirty)/50. then begin
      ind_mapxp=array_indices(dirty_now, maxp  )
      tempc=  reform(this_patt[ind_mapxp[0],ind_mapxp[1],*])* reform( rebin( [dirty_now[maxp]], 1, n_det_new,/sample))

      newd=this_patt*0.
      for ii=0,n_det_new-1 do  newd[*,*,ii]= f_div( the_sf_weight[ii]* tempc[ii] * ( reform(this_patt[*,*,ii])- p_avg), p_avg2)  ;/11.

      newd1=average(newd, 3)

      clean_com[ind_mapxp[0],ind_mapxp[1]]=clean_com[ind_mapxp[0],ind_mapxp[1]] + (dirty_now[maxp]*(gain_fac))
      dirty_now=dirty_now-newd1*gain_fac

    endif else goto, break1

  endfor
  
  break1: 
  res_image = dirty_now ;240115, YS
  cleaned= clean_com*0.
  bw= beam_w/psize   ;240309, YS
  pos_s=where(clean_com gt 0.,npos)
  for xx=0, npos-1,1  do begin
    ind_mapxp=array_indices(clean_com, pos_s[xx]  )

    xax= findgen(nxy) -ind_mapxp[0]
    yax= findgen(nxy) -ind_mapxp[1]
    xgauss = exp(-xax^2. / (2. * (bw)^2.))
    ygauss = exp(-yax^2 /  (2. * (bw)^2.))

    pixo = xgauss # ygauss
    pixo=pixo* clean_com[pos_s[xx]]/total(pixo)

    cleaned=cleaned+ pixo
  endfor

   if pos_s[0] ne -1 then res_image = dirty_now / total( xgauss # ygauss) else res_image = dirty_now*0.  ;240308,YS ;/ pixel^2  
;  Clean_image=f_div(Cleaned+res_image,total( cleaned+res_image ) )*totf[0]  ;240116,YS
  
  Clean_image=f_div(Cleaned,total( cleaned ) )*totf[0]  ;yang,221109
  ;Cleaned_image=clean_com/total(clean_com )*totf[0]
end
