;+
; Mission:
;    ASO-S/HXI
; Purpose:
;     Get combined spectrum for plotting
; Inputs:
;     spectrum   --  Spectrum struct include tags of ut_edges, ct_edges,rcounts, and ltime
;     detectors  --  e.g. detectors=['g1','g2','g3','b1','b2','b4'] or  detectors=[1,1,0,1,1,0,0,1]
; Op. input:
;     ebins     --  Channel energy of output spectrum
; Keyword:
;     /bkg -- if set, only combine bkg detectors\
;     evtype -- output unit form, 0 for counts, 1 for count rate, 2 for flux
;     /for_lc -- if set, returns spec with time
; Outputs:
;     outspec -- combined spectrum, only combine full-open detectors for default.
; 
; History:
; Zhentong Li, 2023/11/04 ;Combine detectors and combine background detectors seperately
;-

Function hxi_combine_det_spec, spec_in, detectors=detectors, bkg=bkg, evtype=evtype, ebins=ebins, for_lc=for_lc

  spec = {UT_EDGES: spec_in.sptimes4fits, $
      CT_EDGES: spec_in.enebins, $
      RCOUNTS : spec_in.spec4fits, $
      LTIME :    spec_in.splivet4fits}

  checkvar,ebins, spec.ct_edges

  det_ind = detectors
  area = hxi_constant(/detector_area)

  n_open = total(detectors[0:2])  ;number of selected full open detectors
  n_back = total(detectors[3:7])  ;number of selected background detectors

  ;User defined energy bands
  IF KEYWORD_SET(EBINS) THEN BEGIN
    hxi_rd_ct_edges, ebins, ebins
    edge_products, ebins,edges_1=ebin1
    ebmm= minmax(ebins)
    IF EBMM(0) LT MIN(SPEC.CT_EDGES) OR EBMM(1) GT MAX(SPEC.CT_EDGES) THEN BEGIN
      PRINT, 'The user EBINS is out of the HXI lev1 data energy range!'
      PRINT, 'The correct energy range is between '+ trim(MIN(spec.ct_edges) )+' and ' + trim( MAX(spec.ct_edges)) + ' keV.'
      MESSAGE, 'Please set correct EBINS ! ',/info
    ENDIF

    ;;weichen ,2023/05/02
    nebins = n_elements(ebin1)-1
    sz1 = size(spec.rcounts)
    neband = sz1[1]
    ndet   = sz1[2]
  if sz1[0] eq 3 then ntime  = sz1[3] else ntime=1

    ocounts= spec.rcounts
    oeb = get_edges(spec.ct_edges,/edges_1)

    Meb = rebin(oeb, 10*n_elements(oeb))
    Mcounts = rebin(ocounts/10.,nebins*10,ndet,ntime,/sample)  ;230602, YS

    outdata = fltarr(nebins, ndet,ntime)
    ix= value_locate(ebin1, meb)
    hist= histogram(ix,min=0,max=nebins,loc=xloc,reverse_indices=R)

    for i=0, nebins-1 do $
      outdata[i,*,*] = total(mcounts[R[R[i]:R[i+1]-1],*,*],1 )
    outltime= spec.ltime

    ;221107, yang
    ndim = (SIZE(spec.ltime))[0]
    IF NDIM EQ 1 THEN BEGIN
      outltime= REBIN( spec.ltime,  (SIZE(spec.ltime))[1], nebins )
      outltime = TRANSPOSE(outltime, [1,0])
    endif
    if ndim eq 2 then begin
      outltime= REBIN( spec.ltime, (SIZE(spec.ltime))[1],(size(spec.ltime))[2], nebins )
      ;;LZT 20231105
      outltime = nebins eq 1 ? reform(outltime, [nebins,(SIZE(spec.ltime))[1],(size(spec.ltime))[2]]) : TRANSPOSE(outltime, [2,0,1])
    endif
  ENDIF ELSE BEGIN
    ebins   = spec.ct_edges
    outdata = spec.rcounts
    outltime= spec.ltime

    ;221107, yang
    ;outltime = reproduce(spec.ltime, (size(ebins))[2])
    ndim = (size(spec.ltime))[0]

    if ndim eq 1 then begin
      outltime= rebin( spec.ltime,  (size(spec.ltime))[1], (size(ebins))[2]  )
      outltime = transpose(outltime, [1,0])
    endif
    if ndim eq 2 then begin
      outltime= rebin( spec.ltime, (size(spec.ltime))[1],(size(spec.ltime))[2], (size(ebins))[2]  )
      outltime = transpose(outltime, [2,0,1])
    endif
  ENDELSE

    tempdata = outdata
    ltime  =  outltime
    detcounts_sum = tempdata[*,0,*]*0
    detrate_sum = detcounts_sum

if ~keyword_set(bkg) then begin
 
;;----- For full open detectors -----
  for j=0, 2 do detcounts_sum = detcounts_sum + tempdata[*,j,*]*det_ind[j]
  for j=0, 2 do detrate_sum = detrate_sum + f_div(tempdata[*,j,*], ltime[*,j,*])*det_ind[j]
  ewid = get_edges(ebins,/width) 
  nebin = n_elements(ewid)
;  livetime_eff = reform(f_div(detcounts_sum, detrate_sum), [nebin,(SIZE(outltime))[3]])  ;; LZT 20231105
  livetime_eff_temp = reform(f_div(total(detcounts_sum,1), total(detrate_sum,1))) ;;LZT 20231114
  livetime_eff = transpose(reproduce(livetime_eff_temp, nebin))
  
  cts=reform(detcounts_sum, [nebin,(SIZE(outltime))[3]] )

if ~keyword_set(evtype) then evtype=0
    case evtype of
      0: cts = keyword_set(for_lc) ? cts : total(cts, 2)
      1: cts = keyword_set(for_lc) ? f_div(cts,livetime_eff) : f_div(total(cts,2),total(livetime_eff,2))
;      1: cts = keyword_set(for_lc) ? f_div(cts,livetime_eff) : average(f_div(cts,livetime_eff),2)
      2: begin
        ctst = cts*0
        if keyword_set(for_lc) then begin 
          for k=0, nebin-1 do ctst[k, *] = f_div(cts[k,*],livetime_eff[k,*])/ewid[k]/area/float(n_open)
          cts=ctst
        endif else begin
          cts = f_div(total(cts,2),total(livetime_eff,2))/ewid/area/float(n_open)
        endelse
      end
    endcase
;stop
  return, {data:cts, splivet4fits:livetime_eff}

endif else begin
  
;;;------- For background detectors -------

  for j=3, 7 do detcounts_sum = detcounts_sum + tempdata[*,j,*]*det_ind[j]
  for j=3, 7 do detrate_sum = detrate_sum + f_div(tempdata[*,j,*], ltime[*,j,*])*det_ind[j]
  ewid = get_edges(ebins,/width)
  nebin = n_elements(ewid)
;  livetime_eff = reform(f_div(detcounts_sum, detrate_sum), [nebin,(SIZE(outltime))[3]])  ;; LZT 20231105
  livetime_eff_temp = reform(f_div(total(detcounts_sum,1), total(detrate_sum,1))) ;;LZT 20231114
  livetime_eff = transpose(reproduce(livetime_eff_temp, nebin))
  cts=reform(detcounts_sum, [nebin,(SIZE(outltime))[3]] )

if ~keyword_set(evtype) then evtype=0
    ewid = get_edges(ebins,/width)
    case evtype of
      0: cts = keyword_set(for_lc) ? cts : total(cts, 2)
      1: cts = keyword_set(for_lc) ? f_div(cts,livetime_eff) : f_div(total(cts,2),total(livetime_eff,2))
      2: begin
        ctst = cts*0
        nebin = n_elements(ewid)
        if keyword_set(for_lc) then begin
          for k=0, nebin-1 do ctst[k, *] = f_div(cts[k,*],livetime_eff[k,*])/ewid[k]/area/float(n_back)
          cts=ctst
        endif else begin
          cts = f_div(total(cts,2),total(livetime_eff,2))/ewid/area/float(n_back)
        endelse
      end
    endcase

  return, {data:cts, splivet4fits:livetime_eff}

endelse

END

