;+
; Mission:
;    ASO-S/HXI
; Purpose:
;     Write spectrum and corresponding DRM files for OSPEX use
; Inputs:
;     spectrum   --  Spectrum struct include tags of ut_edges, ct_edges,rcounts, and ltime 
;     detectors  --  e.g. detectors=['g1','g2','g3','b1','b2','b4'] or  detectors=[1,1,0,1,1,0,0,1]
; Op. input:
;      ebins     --  Channel energy of output spectrum   
; Keyword:
;     combine_check -- array[2], if combine_check[0] equals 1 then combine detectors
;                                if combine_check[1] equals 1 then combine background detectors
; Outputs:
;     specfile   -- 
;     drmfile    --
;     
; History:
;     WeiChen, 2022/04/22 ;Created
;              2022/07/14 ;modify fits header
;              2023/03/20 ;modify default set of detectors;  keep 'back' detectors respon matrix.
;              2023/05/02 ;Modify the processing of keyword Ebins
;              2023/10/20 ;Added option to merge spectra for chosen detectors, Wei Chen & Zhentong Li.
; Zhentong Li, 2023/10/24 ;Spectra of corresponding background detectors will be automatically generated.
; Zhentong Li, 2023/11/03 ;Combine detectors and combine background detectors seperately
;     WeiChen, 2024/01/25 ; add keywords pse (pluse spread effect) for get DRM              

PRO hxi_write_spex_fits, spec,outdir=outdir, ebins=ebins, $
              detectors=detectors, sepdet=sepdet, combine_check=combine_check,$
              pse=pse

checkvar, pse, 1
checkvar,ebins, spec.ct_edges
ebin_mark='_bin'+trim((size(ebins))[2]) ;;bin code mark, LZT 20231020
;select detectors 2023/03/23,chen
default,  detectors, [0,0,1,0,0,0,0,0 ]
det_ind = detectors
odetectors=detectors
;detectors= hxi_det_config(detectors)
area = hxi_constant(/detector_area)

;; 20231020 Zhentong Li
n_det = total(detectors)        ;number of selected detectors
n_thick = total(detectors[0:1]) ;number of selected thick detectors
n_thin = double(detectors[2])   ;number of selected thin detectors
n_open = n_thick + n_thin       ;number of selected full open detectors
n_back = total(detectors[3:7])  ;number of selected background detectors

;;;;===============

;User defined energy bands
IF KEYWORD_SET(EBINS) THEN BEGIN
  hxi_rd_ct_edges, ebins, ebins
  edge_products, ebins,edges_1=ebin1
   ebmm= minmax(ebins)
    IF EBMM(0) LT MIN(SPEC.CT_EDGES) OR EBMM(1) GT MAX(SPEC.CT_EDGES) THEN BEGIN
      PRINT, 'The user EBINS is out of the HXI lev1 data energy range!'
      PRINT, 'The correct energy range is between '+ trim(MIN(spec.ct_edges) )+' and ' + trim( MAX(spec.ct_edges)) + ' keV.'
      MESSAGE, 'Please set correct EBINS ! ',/info
    ENDIF
 
  ;;weichen ,2023/05/02 
  nebins = n_elements(ebin1)-1
  sz1 = size(spec.rcounts)
  neband = sz1[1]
  ndet   = sz1[2]
  if sz1[0] eq 3 then ntime  = sz1[3] else ntime=1
  
  ocounts= spec.rcounts
  oeb = get_edges(spec.ct_edges,/edges_1)
  ;放大十倍
  Meb = rebin(oeb, 10*n_elements(oeb))
  Mcounts = rebin(ocounts/10.,nebins*10,ndet,ntime,/sample)  ;230602, YS

  ;定位，重分配
  outdata = fltarr(nebins, ndet,ntime)
  ix= value_locate(ebin1, meb)
  hist= histogram(ix,min=0,max=nebins,loc=xloc,reverse_indices=R)
  
  for i=0, nebins-1 do $
    outdata[i,*,*] = total(mcounts[R[R[i]:R[i+1]-1],*,*],1 )
    outltime= spec.ltime

     ;221107, yang
     ndim = (SIZE(spec.ltime))[0]
     IF NDIM EQ 1 THEN BEGIN
       outltime= REBIN( spec.ltime,  (SIZE(spec.ltime))[1], nebins )
       outltime = TRANSPOSE(outltime, [1,0])
     endif
     if ndim eq 2 then begin
       outltime= REBIN( spec.ltime, (SIZE(spec.ltime))[1],(size(spec.ltime))[2], nebins )
       outltime = TRANSPOSE(outltime, [2,0,1])
     endif

ENDIF ELSE BEGIN
  ebins   = spec.ct_edges
  outdata = spec.rcounts
  outltime= spec.ltime
  
  ;221107, yang
  ;outltime = reproduce(spec.ltime, (size(ebins))[2])
  ndim = (size(spec.ltime))[0]
 
  if ndim eq 1 then begin
    outltime= rebin( spec.ltime,  (size(spec.ltime))[1], (size(ebins))[2]  )
    outltime = transpose(outltime, [1,0])
  endif
  if ndim eq 2 then begin
    outltime= rebin( spec.ltime, (size(spec.ltime))[1],(size(spec.ltime))[2], (size(ebins))[2]  )
    outltime = transpose(outltime, [2,0,1])
  endif
ENDELSE

nchan = n_elements(ebins) / 2.
ntime = n_elements(spec.ut_edges) / 2.

tr = [spec.ut_edges[0,0], spec.ut_edges[1,ntime-1]]
stim = time2file(tr[0])
etim = time2file(tr[1])
fname='hxi_'+stim+STRMID(etim,8,5)

specnum = indgen(ntime)+1
channel = rebin(lindgen(nchan), nchan, ntime)
timedel = get_edges(spec.ut_edges, /width)
timecen = get_edges(spec.ut_edges, /mean)
;timezero = anytim(spec.ut_edges[0], /MJD )
mjd = anytim(tr, /mjd)
timezero = mjd[0]
timezeri = mjd[0].mjd    ; Integer part of reference time minus mjdref
tstartf = double(mjd[0].time)/8.64d7   ; fractional part of ref time, msec / total msec in day
timezerf = 0.0
; Final time of data collection, integer and fractional parts.
tstopi = mjd[1].mjd
tstopf = double(mjd[1].time) / 8.64d7

rate_struct=hxi_rate_header()
rate_struct.author = 'SPECTRUM2FITS'
rate_struct.detchans = nchan
rate_struct.timezero = timezeri - rate_struct.mjdref
rate_struct.tstarti = timezeri - long(rate_struct.mjdref)
rate_struct.tstartf = tstartf
rate_struct.tstopi = tstopi - long(rate_struct.mjdref)
rate_struct.tstopf = tstopf
rate_struct.telapse = double(((tstopi-timezeri) + (tstopf-tstartf))*86400.0)

fxhmake, hdr, 0, /extend, /date
fxaddpar, hdr, 'DATE_OBS', anytim(tr[0], /ccsds)
fxaddpar, hdr, 'DATE_END', anytim(tr[1], /ccsds)
fxaddpar, hdr, 'ENERGY_L', min(ebins), 'keV'
fxaddpar, hdr, 'ENERGY_H', max(ebins), 'keV'
fxaddpar, hdr, 'TELESCOP', 'ASO-S'
fxaddpar, hdr, 'INSTRUME', 'HXI'
fxaddpar, hdr, 'OBJECT', 'Sun'
primary_hdr = hdr
fxaddpar, primary_hdr, 'COMMENT', 'These FITS files are derived from the lev10 files at '
fxaddpar, primary_hdr, 'COMMENT', 'The files are named hxi_spectrum_yyyymmddd_hhmm_xx.fits'
fxaddpar, primary_hdr, 'COMMENT', ' where yyyymmdd_hhmm is the start time of the data in the file,'

fxbhmake, ext_hdr, ntime, /date

dir= ~keyword_set(outdir)? curdir(): outdir
tempdata = outdata
ltime  =  outltime

;;----- For full open detectors (line 163 to 318)-----
IF n_open ne 0 THEN BEGIN
IF combine_check[0] eq 0 or n_open eq 1 THEN BEGIN

  detectors[3:7] = 0   ;only consider full open
  detid = hxi_det_config(detectors,/id)
  detyp = hxi_det_config(detectors,/type)
  outfiles=concat_dir(dir, fname + ebin_mark + '_'+detid+'_4spex.fits')
  dind= where(detectors eq 1)
  newrcounts= outdata[*,dind,*]
  newltime  = outltime[*,dind,*]

  for i=0, n_elements(detid)-1 do begin
    fxaddpar,primary_hdr,'GEOAREA', area,'Detector area for data in fit results (cm^2)'
    fxaddpar,primary_hdr,'DET_NAM',detid[i],'Detector name'
     CASE detid[i] OF 
      't1' : outdrms = fname + ebin_mark + '_thick_drm.fits'
      't2' : outdrms = fname + ebin_mark + '_thick_drm.fits'
      't3' : outdrms = fname + ebin_mark + '_thin_drm.fits'
      ELSE:  outdrms = fname + ebin_mark + '_bkg_drm.fits'
     ENDCASE
    fxaddpar,primary_hdr,'DRMFILE',outdrms  
    ; 2022/4/9 change unit of detdata as rates
    detdata  = reform(f_div(newrcounts[*,i,*], newltime[*,i,*]))
    livetime = reform(newltime[*,i,*])
    
    spectrum2fits, outfiles[i], /WRITE_PRIMARY_HEADER,  RATE_STRUCT=rate_struct, $ ;
      PRIMARY_HEADER=primary_hdr, EXTENSION_HEADER=ext_hdr, $
      DATA=detdata, $
      UNITS='counts s^-1', SPEC_NUM=specnum, CHANNEL=channel, $
      TIMEDEL=timedel, $
      TIMECEN=timecen, $
      NROWS = N_elements( timecen ), $
      LIVETIME=livetime, NUMBAND=nchan, MINCHAN=Lindgen(nchan), $
      MAXCHAN=Lindgen(nchan)+1, $
      E_MIN=Reform( ebins[0,*] ), $
      E_MAX=Reform( ebins[1,*] ), E_UNIT='keV', $
      ERR_CODE=err_code, _EXTRA=extra_keys, ERR_MSG=err_msg
  endfor

  fxhmake,header,0,/extend,/date
  fxaddpar,header,'TYPE','simulation','RM from Geant4 simulation'
  fxaddpar,header,'TELESCOP','ASO-S','Name of spacescraft' ;240308,YS
  fxaddpar,header,'INSTRUME','HXI', 'Name of instrument'
  primary_hdr= header
  fxaddpar, primary_hdr, 'COMMENT', 'These FITS files are derived from the lev10 files '
  
  ss = detyp
  FOR i=0 , N_ELEMENTS(ss)-1 DO BEGIN
    CASE ss[i] OF
      'thick': BEGIN
        hxi_get_srm4user, ebins, thick_drm, /linear,pho_ener=phbins,pse=pse
        outdrms=concat_dir(dir,fname + ebin_mark + '_'+ss[i]+'_drm.fits')
        ph_edge= get_edges(phbins,/edges_2)
        ct_edge= get_edges(ebins,/edges_2)
        fxaddpar,primary_hdr,'EXPLAIN','resp_v9_0p85mm.sav'
        fxaddpar,primary_hdr,'GEOAREA', area,'Detector area for data in fit results (cm^2)'
        namdet=['T1','T2']
        indet=WHERE(detectors[0:1] EQ 1)
        detnam=namdet[indet[0]]
        fxaddpar,primary_hdr,'DET_NAM', detnam, 'Detector name'
        fxaddpar,primary_hdr,'DET_TYP',ss[i],' '
        fxaddpar,primary_hdr,'PSE', trim(pse), 'Pluse spread effect for drm. 0:close, 1:open'
        fxaddpar,primary_hdr,'DETCHANS',n_elements(ct_edge)/2,'Total number of detector HXI channels'
        fxaddpar,primary_hdr,'CREATOR','HXI-team',''

        rm2fits,outdrms,ph_edge,thick_drm,/write_primary_header, $
          primary_header=primary_hdr,det_channels=ct_edge, $
          /rmf_header
      END

      'thin': BEGIN
        hxi_get_srm4user, ebins, thin_drm, /linear,pho_ener=phbins ,/thin_det,pse=pse
        outdrms=concat_dir(dir,fname + ebin_mark + '_'+ss[i]+'_drm.fits')
        ph_edge= get_edges(phbins,/edges_2)
        ct_edge= get_edges(ebins,/edges_2)
        FxAddPar,primary_hdr,'EXPLAIN','resp_v9_0p50mm.sav'
        fxaddpar,primary_hdr,'GEOAREA', area,'Detector area for data in fit results (cm^2)'
        fxaddpar,primary_hdr,'DET_TYP',ss[i],''
        fxaddpar,primary_hdr,'DET_NAM','T3',''
        fxaddpar,primary_hdr,'PSE', trim(pse), 'Pluse spread effect for drm. 0:close, 1:open'
        fxaddpar,primary_hdr,'DETCHANS',N_ELEMENTS(ct_edge)/2,'Total number of detector HXI channels'
        fxaddpar,primary_hdr,'CREATOR','HXI-team',''

        rm2fits,outdrms,ph_edge,thin_drm,/write_primary_header, $
          primary_header=primary_hdr,  det_channels=ct_edge, $
          /rmf_header
      END
      ELSE: print, 'Please choose correct detectors.'
    ENDCASE
  ENDFOR
  
ENDIF ELSE BEGIN

  ;;;==================================
  ;;--- combine selected open detectors

  detid = (hxi_det_config(det_ind,/id))[0:n_open-1]
  outfiles=concat_dir(dir,fname + ebin_mark + '_'+strjoin(detid)+'_4spex.fits')
  fxaddpar,primary_hdr,'DET_TYP','CDET','Combined-Detectors'
  
  detcounts_sum = tempdata[*,0,*]*0
  detrate_sum = detcounts_sum
  
  for j=0, 2 do detcounts_sum = detcounts_sum + tempdata[*,j,*]*det_ind[j]
  for j=0, 2 do detrate_sum = detrate_sum + f_div(tempdata[*,j,*], ltime[*,j,*])*det_ind[j]
  
;  livetime_eff0 = reform(f_div(detcounts_sum, detrate_sum))  ;; LZT 20231026
  livetime_eff_temp = reform(f_div(total(detcounts_sum,1), total(detrate_sum,1))) ;;LZT 20231114
  livetime_eff = transpose(CMREPLICATE(livetime_eff_temp, nchan))
  detdata = reform(detrate_sum)
;  if keyword_set(for_plot) then begin
;    detcounts_sum=reform(detcounts_sum)
;    combine_plot= {data:detcounts_sum, splivet4fits:livetime_eff}
;  endif  
    
  fxaddpar,primary_hdr,'DRMFILE',fname + ebin_mark + '_'+strjoin(detid)+'_drm.fits'
  fxaddpar,primary_hdr,'GEOAREA', double(n_open)* area,'Detector area for data in fit results (cm^2)' ;; LZT 0729
  fxaddpar,primary_hdr,'DET_NAM', strjoin(STRUPCASE(detid),'+'), 'Detector name' ;;LZT 0729

  spectrum2fits, outfiles,  /write_primary_header,  rate_struct=rate_struct,$ ;
    primary_header=primary_hdr, EXTENSION_HEADER=ext_hdr, $
    DATA=detdata, $
    UNITS='counts s^-1', SPEC_NUM=specnum, CHANNEL=channel, $
    TIMEDEL=timedel, $
    TIMECEN=timecen, $
    NROWS = N_ELEMENTS( timecen ), $
    LIVETIME=livetime_eff, NUMBAND=nchan, MINCHAN=LINDGEN(nchan), $
    MAXCHAN=LINDGEN(nchan)+1, $
    E_MIN=REFORM( ebins[0,*] ), $
    E_MAX=REFORM( ebins[1,*] ), E_UNIT='keV', $
    ERR_CODE=err_code, _EXTRA=extra_keys, ERR_MSG=err_msg

  ;;  --- write combined open det drm ---

  fxhmake,header,0,/extend,/date
  fxaddpar,header,'TYPE','simulation','RM from Geant4 simulation'
  fxaddpar,header,'TELESCOP','ASO-S','Name of spacescraft
  fxaddpar,header,'INSTRUME','HXI', 'Name of instrument'
  primary_hdr= header
  fxaddpar, primary_hdr, 'COMMENT', 'These FITS files are derived from the lev10 files '
  hxi_get_srm4user, ebins, thick_drm, /linear,pho_ener=phbins ,pse=pse
  hxi_get_srm4user, ebins, thin_drm, /linear,pho_ener=phbins ,/thin_det,pse=pse
  mdrm = (n_thick*thick_drm + n_thin*thin_drm) / double(n_open)  ;LZT, 20231020
  outdrms= concat_dir(dir, fname + ebin_mark + '_'+strjoin(detid)+'_drm.fits')
  ph_edge= get_edges(phbins,/edges_2)
  ct_edge= get_edges(ebins,/edges_2)
  fxaddpar,primary_hdr,'EXPLAIN','Combined DRM'
  fxaddpar,primary_hdr,'GEOAREA', double(n_open) * area,'Detector area for data in fit results (cm^2)' ;LZT, 20231020
  fxaddpar,primary_hdr,'DET_NAM', strjoin(detid,'+'), 'Detector name' ;LZT, 20231020
  fxaddpar,primary_hdr,'DET_TYP','CDET','Combined-Detectors DRM'
  fxaddpar,primary_hdr,'PSE', trim(pse), 'Pluse spread effect for drm. 0:close, 1:open'
  fxaddpar,primary_hdr,'DETCHANS',n_elements(ct_edge)/2,'Total number of detector HXI channels'
  fxaddpar,primary_hdr,'CREATOR','HXI-team',''

  rm2fits,outdrms,ph_edge,mdrm,/write_primary_header, $
    primary_header=primary_hdr,det_channels=ct_edge, $
    /rmf_header, /SILENT
ENDELSE
ENDIF

;;;------- For background detectors -------
if n_back ne 0 then begin
if combine_check[1] eq 0 or n_back eq 1 then begin
  detectors = odetectors
  detectors[0:2] = 0   ;only consider background detectors
  detid = hxi_det_config(detectors,/id)    ;b1 to b5
  detyp = hxi_det_config(detectors,/type)  ;bkg
  outfiles=concat_dir(dir, fname + ebin_mark + '_'+detid+'_4spex.fits')
  dind= where(detectors eq 1)
  newrcounts= outdata[*,dind,*]
  newltime  = outltime[*,dind,*]
  outdrms= concat_dir(dir, fname + ebin_mark + '_bkg_drm.fits')
  for i=0, n_elements(detid)-1 do begin
    detdata  = reform(f_div(newrcounts[*,i,*], newltime[*,i,*]))
    livetime = reform(newltime[*,i,*])
    fxaddpar,primary_hdr,'GEOAREA', area,'Detector area for data in fit results (cm^2)'
    fxaddpar,primary_hdr,'DET_NAM',detid[i],'Detector name'
    fxaddpar,primary_hdr,'DRMFILE',fname + ebin_mark + '_bkg_drm.fits'
    spectrum2fits, outfiles[i], /WRITE_PRIMARY_HEADER,  RATE_STRUCT=rate_struct, $ ;
      PRIMARY_HEADER=primary_hdr, EXTENSION_HEADER=ext_hdr, $
      DATA=detdata, $
      UNITS='counts s^-1', SPEC_NUM=specnum, CHANNEL=channel, $
      TIMEDEL=timedel, $
      TIMECEN=timecen, $
      NROWS = N_elements( timecen ), $
      LIVETIME=livetime, NUMBAND=nchan, MINCHAN=Lindgen(nchan), $
      MAXCHAN=Lindgen(nchan)+1, $
      E_MIN=Reform( ebins[0,*] ), $
      E_MAX=Reform( ebins[1,*] ), E_UNIT='keV', $
      ERR_CODE=err_code, _EXTRA=extra_keys, ERR_MSG=err_msg
  endfor
  
  hxi_get_srm4user, ebins, back_drm, /linear,pho_ener=phbins,pse=pse
  ph_edge= get_edges(phbins,/edges_2)
  ct_edge= get_edges(ebins,/edges_2)
  fxhmake,header,0,/extend,/date
  fxaddpar,header,'TYPE','simulation','RM from Geant4 simulation'
  fxaddpar,header,'TELESCOP','ASO-S','Name of spacescraft' ;240308
  fxaddpar,header,'INSTRUME','HXI', 'Name of instrument'
  primary_hdr= header
  fxaddpar, primary_hdr, 'COMMENT', 'These FITS files are derived from the lev10 files '
  fxaddpar,primary_hdr,'EXPLAIN','Background DRM'
  fxaddpar,primary_hdr,'GEOAREA', area,'Detector area for data in fit results (cm^2)'
  fxaddpar,primary_hdr,'DET_NAM', 'BKG', 'Detector name' 
  fxaddpar,primary_hdr,'DET_TYP','BKG','Background-Detectors DRM'
  fxaddpar,primary_hdr,'PSE', trim(pse), 'Pluse spread effect for drm. 0:close, 1:open'
  fxaddpar,primary_hdr,'DETCHANS',n_elements(ct_edge)/2,'Total number of detector HXI channels'
  fxaddpar,primary_hdr,'CREATOR','HXI-team',''

  rm2fits,outdrms,ph_edge,back_drm,/write_primary_header, $
    primary_header=primary_hdr,det_channels=ct_edge, $
    /rmf_header, /SILENT
endif else begin
  ;;;==========================
  ;;--- combine bkg detectors
  detbkid = (hxi_det_config(det_ind,/id))[n_open:n_open+n_back-1]
  outfiles=concat_dir(dir,fname + ebin_mark + '_'+strjoin(detbkid)+'_4spex.fits')
  
  detcounts_sum = tempdata[*,0,*]*0
  detrate_sum = detcounts_sum

  for j=3, 7 do detcounts_sum = detcounts_sum + tempdata[*,j,*]*det_ind[j]  
  for j=3, 7 do detrate_sum = detrate_sum + f_div(tempdata[*,j,*], ltime[*,j,*])*det_ind[j]

;  livetime_eff = reform(f_div(detcounts_sum, detrate_sum))  ;; LZT 20231026
  livetime_eff_temp = reform(f_div(total(detcounts_sum,1), total(detrate_sum,1))) ;;LZT 20231114
  livetime_eff = transpose(CMREPLICATE(livetime_eff_temp, nchan))
  detdata = reform(detrate_sum)
  
  fxaddpar,primary_hdr,'DET_TYP','CBKG','Combined-BKG-Detectors'
  fxaddpar,primary_hdr,'DRMFILE',fname + ebin_mark + '_'+strjoin(detbkid)+'_drm.fits'
  fxaddpar,primary_hdr,'GEOAREA', double(n_back)* area,'Detector area for data in fit results (cm^2)' ;; LZT 0729
  fxaddpar,primary_hdr,'DET_NAM', strjoin(STRUPCASE(detbkid),'+'), 'Detector name' ;;LZT 0729

  spectrum2fits, outfiles,  /write_primary_header,  rate_struct=rate_struct,$ ;
    primary_header=primary_hdr, EXTENSION_HEADER=ext_hdr, $
    DATA=detdata, $
    UNITS='counts s^-1', SPEC_NUM=specnum, CHANNEL=channel, $
    TIMEDEL=timedel, $
    TIMECEN=timecen, $
    NROWS = N_ELEMENTS( timecen ), $
    LIVETIME=livetime_eff, NUMBAND=nchan, MINCHAN=LINDGEN(nchan), $
    MAXCHAN=LINDGEN(nchan)+1, $
    E_MIN=REFORM( ebins[0,*] ), $
    E_MAX=REFORM( ebins[1,*] ), E_UNIT='keV', $
    ERR_CODE=err_code, _EXTRA=extra_keys, ERR_MSG=err_msg
  ;;  --- write combined bkg drm ---
;  stop
  hxi_get_srm4user, ebins, back_drm, /linear,pho_ener=phbins,pse=pse
  fxhmake,header,0,/extend,/date
  fxaddpar,header,'TYPE','simulation','RM from Geant4 simulation'
  fxaddpar,header,'TELESCOP','ASO-S','Name of spacescraft
  fxaddpar,header,'INSTRUME','HXI', 'Name of instrument'
  primary_hdr= header
  fxaddpar, primary_hdr, 'COMMENT', 'These FITS files are derived from the lev10 files '
  ph_edge= get_edges(phbins,/edges_2)
  ct_edge= get_edges(ebins,/edges_2)
  outdrms= concat_dir(dir, fname + ebin_mark + '_'+strjoin(detbkid)+'_drm.fits')
  fxaddpar,primary_hdr,'EXPLAIN','Combined BKG DRM'
  fxaddpar,primary_hdr,'GEOAREA', double(n_back) * area,'Detector area for data in fit results (cm^2)' ;LZT, 20231020
  fxaddpar,primary_hdr,'DET_NAM', strjoin(detbkid,'+'), 'Detector name' ;LZT, 20231020
  fxaddpar,primary_hdr,'DET_TYP','CBKG','Combined-BKG-Detectors DRM'
  fxaddpar,primary_hdr,'PSE', trim(pse), 'Pluse spread effect for drm. 0:close, 1:open'
  fxaddpar,primary_hdr,'DETCHANS',n_elements(ct_edge)/2,'Total number of detector HXI channels'
  fxaddpar,primary_hdr,'CREATOR','HXI-team',''

  rm2fits,outdrms,ph_edge,back_drm,/write_primary_header, $
    primary_header=primary_hdr,det_channels=ct_edge, $
    /rmf_header
endelse
endif

END